from gtkmvc.model import Model
import os
import os.path
import statvfs
import re
import gtk
from popen2 import Popen3, Popen4
from select import select
from fcntl import fcntl, F_SETFL


device = "/dev/hdc"
cdrecord = "/usr/bin/cdrecord "
mkisofs = "/usr/bin/mkisofs "
cdrdao = "/usr/bin/cdrdao "
tmpdir = "/tmp"
tmpimage = os.path.join(tmpdir, "birne.iso")


def check_media_empty():
    """check if cdrom is empty using cdrdao"""
    # taken almost verbatim from Gaetano Paolone's burn v0.4.3
    # giuppi's function 
    media_raw_info=popen2.popen2('%s disk-info --device %s 2>/dev/null'%(cdrdao,device))
    lines = media_raw_info[0].readlines()
    for line in lines:
        if line.startswith('CD-R empty'):
            if line.split(':')[1].strip() == 'yes':
                return True
            return False
#            while gtk.events_pending():
#                gtk.main_iteration()
    return False

def get_media_capacity():
    """get media capacity using cdrdao"""
    # taken almost verbatim from Gaetano Paolone's burn v0.4.3
    # giuppi's function 
    media_raw_info = popen2.popen2('%s disk-info --device %s 2>/dev/null'%(cdrdao,device))
    lines = media_raw_info[0].readlines()
    for line in lines:
        if line.startswith('Total Capacity'):
            if line.split(':')[1].strip() == 'n/a':
                return None
            return line.split()[6].split('/')[0]
    return None


def du(dirname):
    # taken almost verbatim from Gaetano Paolone's burn v0.4.3
    """Goes deeply through path in order to calculate path's disk usage."""
    path_du = 0
    for root, dirs, files in os.walk(dirname):
        for i in files:
            if os.path.exists(os.path.realpath(os.path.join(root, i))):
                path_du = path_du + os.path.getsize(os.path.join(root, i))
            while gtk.events_pending():
                gtk.main_iteration()
    #if options.follow_symlink: #only if user puts -l on command line
    #    for d in dirs:
    #        print join(root, d)
    #        #os.walk does not follow symlink directories. Next line will allow it.
    #        if islink(join(root, d)):
    #            path_du = path_du + du(realpath(join(root,d)))
    return path_du

def get_target(absfile, is_relative, strip_path):
    """ Returns the target filename from an absolute source name. """
    relfile = absfile.split(strip_path, 1)[1]
    return ( is_relative and relfile ) or file



class BirneModel (Model):
    """ Model for Birne's main GUI. It contains everything necessary to
    build various types of CDs."""

    status_messages = ( 'IDLE',    'BURNING', 'FIXATING',
                        'READING', 'MKISOFS', 'WAITING'  )

    # observable properties:
    __properties__ = {
        "mode"          : "copy",    # type of disc; must be one of "copy", "audio",
                                     # "iso" or "files"
        "speed"         : 4,         # burning speed
        "simulate"      : True,      # burn in dummy mode?
        "eject"         : True,      # eject when finished?
        "content"       : {},        # content of disc. keys are target
                                     # paths, values are tuples of source paths
                                     # and size
        "contentsize"   : 0,         # size of disc content
        "status"        : "",        # status message
        "progress"      : (),        # progress of current action
        "discfree"      : 0          # free space on partition where tmpdir
                                     # resides
        }


    def __init__(self):
        Model.__init__(self)
        pass

    def cleanup(self):
        """Method to call before exiting the program. Currently deletes all
        temporary files that have been created."""
        if os.path.exists(tmpimage):
            os.remove(tmpimage)
        pass

    def burn(self):
        """Executes the burning function appropriate for current mode."""
        method = getattr(self, "burn_%s" % self.mode, "burn_copy")
        return method()

    def burn_copy(self):
        pass

    def burn_audio(self):
        pass

    def burn_files(self, burn_files=True):
        """Creates an ISO image from self.content with mkisofs and burns it
        to CD."""
        mkisofs_line = "%s -o %s -J -R -gui -graft-points " \
                       % (mkisofs, tmpimage)
        graft_points = " ".join(["%s=%s" % 
            (target, source[0]) for target, source in self.content.items()])
        mkisofs_line += graft_points
        if os.path.exists(tmpimage): os.remove(tmpimage)
        self.run_mkisofs(mkisofs_line)
        if burn_files: self.run_cdrecord(tmpimage);
        pass

    def burn_image(self):
        """Burns a previously compiled image. Its source has to be the only
        element in self.content."""
        #assert ( len(self.content) == 1 )
        imagefile = " ".join( [self.content[key][0] for key in self.content] )
        self.run_cdrecord(imagefile);
        pass

    def add_files(self, new_files, is_relative=True, strip_path=""):
        """Adds list of files and directories to self.content. Dictionary keys
        are the destination paths (because the have to be unique) and
        dictionary values are tuples of source path and (size or length).
        Destination paths differ from source paths if is_relative is set True.
        In this case, strip_path will be truncated from the source path."""

        # we're reading
        self.status = BirneModel.status_messages[3]

        # Get a copy of current CD content...
        new_content = self.content.copy()
        new_contentsize = self.contentsize

        ## ... get new files...
        for file in new_files:
            if os.path.exists(file):
                target = get_target(file, is_relative, strip_path)
                while gtk.events_pending():
                    gtk.main_iteration()
                if target not in self.content:
                    size = du(file)
                    new_contentsize += size
                    new_content[target] = file, size

        # ... and copy new_content back to current content. We need to directly
        # assign the new content to our observable property to trigger
        # notification messages to our observers!
        self.content = new_content.copy()
        self.contentsize = new_contentsize

        # while we're at it, update free space on disc
        stat = os.statvfs(tmpdir)
        self.discfree = (long(stat[statvfs.F_BAVAIL]) * long(stat[statvfs.F_BSIZE]))

        # we're idle again
        self.status = BirneModel.status_messages[0]

        pass


    def run_cdrecord(self, imagefile):
        self.status = BirneModel.status_messages[1]
        command = "%s -v -pad -tao speed=%s dev=%s driveropts=burnfree " \
                         % (cdrecord, self.speed, device)
        if self.simulate: command += "-dummy "
        if self.eject: command += "-eject "
        command += "-data %s" % imagefile
        process = Popen3(command.split(), 1, 10)
        # Non-blocking, please!
        fcntl(process.fromchild.fileno(), F_SETFL, os.O_NONBLOCK)
        fcntl(process.tochild.fileno(),   F_SETFL, os.O_NONBLOCK)
        fcntl(process.childerr.fileno(),  F_SETFL, os.O_NONBLOCK)

        progress_pattern = 'Track \d+:\s+(\d+) of\s+(\d+) MB written'
        fixating_pattern = 'Fixating\.\.\.'
        progress_re = re.compile(progress_pattern)
        fixating_re = re.compile(fixating_pattern)
        # parse input from process while it is running
        while 1:
            (r, w, e) = select([process.fromchild, ],
                               [],
                               [process.childerr, ], 0.1)
            for i in r:
                while gtk.events_pending():
                    gtk.main_iteration()
                proc_output = process.fromchild.read()
                if proc_output:
                    if re.search(progress_pattern, proc_output):
                        self.progress = progress_re.search(proc_output).groups()
                    elif re.search(fixating_pattern, proc_output):
                        self.status = BirneModel.status_messages[2]
            while gtk.events_pending():
                gtk.main_iteration()
            self.retval = process.poll()
            if self.retval != -1: break 
        self.status = BirneModel.status_messages[0]

    def run_mkisofs(self, command):
        # we're creating an ISO image
        self.status = BirneModel.status_messages[4]
        process = Popen4(command.split(), 1)
        # Non-blocking, please!
        fcntl(process.fromchild.fileno(), F_SETFL, os.O_NONBLOCK)
        fcntl(process.tochild.fileno(),   F_SETFL, os.O_NONBLOCK)
        #fcntl(process.childerr.fileno(),  F_SETFL, os.O_NONBLOCK)

        progress_pattern = ' +(\d+)\.\d+. done'
        progress_re = re.compile(progress_pattern)

        while 1:
            (r, w, e) = select([process.fromchild, ], [], [], 0.1)
            for i in r:
                proc_output = process.fromchild.read()
                if proc_output:
                    match = re.search(progress_pattern, proc_output)
                    if match:
                        cursize = self.contentsize * int( match.groups()[0] ) / 100.0
                        cursize = cursize / 1024 / 1024
                        totalsize = self.contentsize / 1024 / 1024
                        self.progress = cursize, totalsize
            while gtk.events_pending():
                gtk.main_iteration()
            self.retval = process.poll()
            if self.retval != -1:
                break

        # we're idle again
        self.status = BirneModel.status_messages[0]
        pass

# vim: expandtab ts=4
